@@ -2,13 +2,14 @@ package Plack::App::MCCS;
 
 # ABSTRACT: Minify, Compress, Cache-control and Serve static files from Plack applications
 
-our $VERSION = "0.007002";
+our $VERSION = "1.000000";
 $VERSION = eval $VERSION;
 
 use strict;
 use warnings;
 use parent qw/Plack::Component/;
 
+use autodie;
 use Cwd ();
 use Fcntl qw/:flock/;
 use File::Spec::Unix;
@@ -17,7 +18,7 @@ use Module::Load::Conditional qw/can_load/;
 use Plack::MIME;
 use Plack::Util;
 
-use Plack::Util::Accessor qw/root defaults types encoding _can_minify_css _can_minify_js _can_gzip/;
+use Plack::Util::Accessor qw/root defaults types encoding _can_minify_css _can_minify_js _can_gzip min_cache_dir/;
 
 =head1 NAME
 
@@ -45,6 +46,7 @@ L<Plack::Component>
 	builder {
 		mount '/static' => Plack::App::MCCS->new(
 			root => '/path/to/static_files',
+			min_cache_dir => 'min_cache',
 			defaults => {
 				valid_for => 86400,
 				cache_control => ['private'],
@@ -214,6 +216,10 @@ If a request comes to C<style.min.css> (which you don't really want when
 using C<MCCS>), the app will not attempt to minify it again (so you won't
 get things like C<style.min.min.css>).
 
+If C<min_cache_dir> is specified, it will do all its searching and storing of
+generated minified files within C<root>/C<$min_cache_dir> and ignore minified
+files outside that directory.
+
 =item 4. Compression
 
 If the client supports gzip encoding (deflate to be added in the future, probably),
@@ -323,6 +329,12 @@ header, C<MCCS> will ignore it).
 
 =back
 
+B<min_cache_dir> - For unminified files, by default minified files are generated
+in the same directory as the original file. If this attribute is specified they
+are instead generated within C<root>/C<$min_cache_dir>, and minified files
+outside that directory are ignored, unless requested directly. This can make it
+easier to filter out generated files when validating a deployment.
+
 Giving C<minify>, C<compress> and C<etag> false values is useful during
 development, when you don't want your project to be "polluted" with all
 those .gz, .min and .etag files.
@@ -403,13 +415,14 @@ sub call {
 	if ($file !~ m/\.min\.(css|js)$/ && ($content_type eq 'text/css' || $content_type eq 'application/javascript')) {
 		my $new = $file;
 		$new =~ s/\.(css|js)$/.min.$1/;
+		$new = $self->_filename_in_min_cache_dir($new) if $self->min_cache_dir;
 		my $min = $self->_locate_file($new);
 
 		my $try_to_minify; # initially undef
 		if ($min && !ref $min) {
 			# yes, we found it, but is it still fresh? let's see
 			# when was the source file modified and compare them
-			
+
 			# $slm = source file last modified date
 			my $slm = (stat(($self->_full_path($file))[0]))[9];
 			# $mlm = minified file last modified date
@@ -440,11 +453,10 @@ sub call {
 					|| return $self->return_403;
 
 				# add ->path attribute to the file handle
-				Plack::Util::set_io_path($ifh, Cwd::realpath($file));
+				Plack::Util::set_io_path($ifh, Cwd::realpath($orig));
 
 				# read the file's contents into $css
 				my $body; Plack::Util::foreach($ifh, sub { $body .= $_[0] });
-				close $ifh;
 
 				# minify contents
 				my $min = $content_type eq 'text/css' ? CSS::Minifier::XS::minify($body) : JavaScript::Minifier::XS::minify($body);
@@ -472,7 +484,7 @@ sub call {
 			# good, we found a compressed version, but is it
 			# still fresh? like before let's compare its modification
 			# date with the current file marked for serving
-			
+
 			# $slm = source file last modified date
 			my $slm = (stat(($self->_full_path($file))[0]))[9];
 			# $clm = compressed file last modified date
@@ -536,6 +548,15 @@ sub _locate_file {
 	}
 }
 
+sub _filename_in_min_cache_dir {
+	my ($self, $file) = @_;
+	my $min_cache_dir = File::Spec->catfile($self->root||".", $self->min_cache_dir);
+	mkdir $min_cache_dir if !-d $min_cache_dir;
+	$file =~ s@/@%2F@g;
+	my $new = File::Spec::Unix->catfile($self->min_cache_dir, $file);
+	return $new;
+}
+
 sub _determine_content_type {
 	my ($self, $file) = @_;
 
@@ -779,7 +800,7 @@ probably means C<MCCS> has no read permissions for that file.
 =back
 
 =head1 CONFIGURATION AND ENVIRONMENT
-  
+
 C<Plack::App::MCCS> requires no configuration files or environment variables.
 
 =head1 DEPENDENCIES
@@ -845,13 +866,15 @@ Ido Perlmuter <ido@ido50.net>
 Some of this module's code is based on L<Plack::App::File> by Tatsuhiko Miyagawa
 and L<Plack::Middleware::ETag> by Franck Cuny.
 
+Christian Walde contributed new features and fixes for the 1.0.0 release.
+
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2011-2015, Ido Perlmuter C<< ido@ido50.net >>.
+Copyright (c) 2011-2016, Ido Perlmuter C<< ido@ido50.net >>.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself, either version
-5.8.1 or any later version. See L<perlartistic|perlartistic> 
+5.8.1 or any later version. See L<perlartistic|perlartistic>
 and L<perlgpl|perlgpl>.
 
 The full text of the license can be found in the
